# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
# Copyright (C) 2015       Lajos Nemeséri <nemeseril@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Hungarian-specific classes for parsing and displaying dates.
"""

#-------------------------------------------------------------------------
#
# Python modules
#
#-------------------------------------------------------------------------
import re

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler

#-------------------------------------------------------------------------
#
# Hungarian parser
#
#
#-------------------------------------------------------------------------
class DateParserHU(DateParser):


    month_to_int = DateParser.month_to_int

    month_to_int["-"]=0   #to make the Zero month to work

    month_to_int["január"] = 1
    month_to_int["jan"] = 1
    month_to_int["jan."] = 1
    month_to_int["I"] = 1
#    month_to_int["i"] = 1

    month_to_int["február"] = 2
    month_to_int["feb"] = 2
    month_to_int["feb."] = 2
    month_to_int["II"] = 2
#    month_to_int["ii"] = 2

    month_to_int["március"] = 3
    month_to_int["márc"] = 3
    month_to_int["márc."] = 3
    month_to_int["III"] = 3
#    month_to_int["iii"] = 3

    month_to_int["április"] = 4
    month_to_int["ápr"] = 4
    month_to_int["ápr."] = 4
    month_to_int["IV"] = 4
#    month_to_int["iv"] = 4

    month_to_int["május"] = 5
    month_to_int["máj"] = 5
    month_to_int["máj."] = 5
    month_to_int["V"] = 5
#    month_to_int["v"] = 5

    month_to_int["június"] = 6
    month_to_int["jún"] = 6
    month_to_int["jún."] = 6
    month_to_int["VI"] = 6
#    month_to_int["vi"] = 6

    month_to_int["július"] = 7
    month_to_int["júl"] = 7
    month_to_int["júl."] = 7
    month_to_int["VII"] = 7
#    month_to_int["vii"] = 7

    month_to_int["augusztus"] = 8
    month_to_int["aug"] = 8
    month_to_int["aug."] = 8
    month_to_int["VIII"] = 8
#    month_to_int["viii"] = 8

    month_to_int["szeptember"] = 9
    month_to_int["szept"] = 9
    month_to_int["szept."] = 9
    month_to_int["IX"] = 9
#    month_to_int["ix"] = 9

    month_to_int["október"] = 10
    month_to_int["okt"] = 10
    month_to_int["okt."] = 10
    month_to_int["X"] = 10
#    month_to_int["x"] = 10

    month_to_int["november"] = 11
    month_to_int["nov"] = 11
    month_to_int["nov."] = 11
    month_to_int["XI"] = 11
#    month_to_int["xi"] = 11

    month_to_int["december"] = 12
    month_to_int["dec"] = 12
    month_to_int["dec."] = 12
    month_to_int["XII"] = 12
#    month_to_int["xii"] = 12


#-----------------------------------------------------------------------
#
#  Alternative and latin names - not verified
#
#-----------------------------------------------------------------------
    # Other common latin names

#    month_to_int["januaris"] = 01
#    month_to_int["januarii"] = 01
#    month_to_int["januarius"] = 01
#    month_to_int["februaris"] = 02
#    month_to_int["februarii"] = 02
#    month_to_int["februarius"] = 02
#    month_to_int["martii"] = 03
#    month_to_int["martius"] = 03
#    month_to_int["aprilis"] = 04
#    month_to_int["maius"] = 05
#    month_to_int["maii"] = 05
#    month_to_int["junius"] = 06
#    month_to_int["junii"] = 06
#    month_to_int["julius"] = 07
#    month_to_int["julii"] = 07
#    month_to_int["augustus"] = 08
#    month_to_int["augusti"] = 08
#    month_to_int["septembris"] = 09
#    month_to_int["7bris"] = 09
#    month_to_int["september"] = 09
#    month_to_int["october"] = 10
#    month_to_int["octobris"] = 10
#    month_to_int["8bris"] = 10
#    month_to_int["novembris"] = 11
#    month_to_int["9bris"] = 11
#    month_to_int["november"] = 11
#    month_to_int["decembris"] = 12
#    month_to_int["10bris"] = 12
#    month_to_int["xbris"] = 12
#    month_to_int["december"] = 12

    # old Hungarian names

#    month_to_int["Boldogasszony hava"] = 01
#    month_to_int["Fergeteg hava"] = 01
#    month_to_int["Böjtelő hava"] = 02
#    month_to_int["Jégbontó hava"] = 02
#    month_to_int["Böjtmás hava"] = 03
#    month_to_int["Kikelet hava"] = 03
#    month_to_int["Szent György hava"] = 04
#    month_to_int["Szelek hava"] = 04
#    month_to_int["Pünkösd hava"] = 05
#    month_to_int["Ígéret hava"] = 05
#    month_to_int["Szent Iván hava"] = 06
#    month_to_int["Napisten hava"] = 06
#    month_to_int["Szent Jakab hava"] = 07
#    month_to_int["Áldás hava"] = 07
#    month_to_int["Kisasszony hava"] = 08
#    month_to_int["Újkenyér hava"] = 08
#    month_to_int["Szent Mihály hava"] = 09
#    month_to_int["Földanya hava"] = 09
#    month_to_int["Mindszent hava"] = 10
#    month_to_int["Magvető hava"] = 10
#    month_to_int["Szent András hava"] = 11
#    month_to_int["Enyészet hava"] = 11
#    month_to_int["Karácsony hava"] = 12
#    month_to_int["Álom hava"] = 12

    modifier_after_to_int={
        'előtt'      : Date.MOD_BEFORE,
        'körül'      : Date.MOD_ABOUT,
        'után'       : Date.MOD_AFTER,
        }

    quality_to_int = {
        'becsült'       : Date.QUAL_ESTIMATED,
        'hozzávetőleg'  : Date.QUAL_ESTIMATED,
        'becs.'         : Date.QUAL_ESTIMATED,
        'számított'     : Date.QUAL_CALCULATED,
        'körülbelül'    : Date.QUAL_ESTIMATED,
        'számolt'       : Date.QUAL_CALCULATED,
        'szám.'         : Date.QUAL_CALCULATED,
        }

    bce = ["időszámításunk előtt", "időszámítás előtt", "i. e.",
           "Krisztus előtt", "Krisztus előtti", "Kr. e."] + DateParser.bce


    calendar_to_int = {
        'Gergely'              : Date.CAL_GREGORIAN,
        'Julián'               : Date.CAL_JULIAN,
        'héber'                : Date.CAL_HEBREW,
        'iszlám'               : Date.CAL_ISLAMIC,
        'francia köztársasági' : Date.CAL_FRENCH,
        'perzsa'               : Date.CAL_PERSIAN,
        'svéd'                 : Date.CAL_SWEDISH,
        }


    def init_strings(self):
        # Compiles regular expression strings for matching dates
        DateParser.init_strings(self)

        self._numeric = re.compile(
            r"((\d+)[/\.])?\s*((\d+)[/\.])?\s*(\d+)[/\. ]?$")
        # this next RE has the (possibly-slashed) year at the string's start
        self._text2 = re.compile(r'((\d+)(/\d+)?\.)?\s+?%s\.?\s*(\d+\.)?\s*$'
                                 % self._mon_str, re.IGNORECASE)
        _span_1 = [r'-tó\(ő\)l', '-tól', '-től']
        _span_2 = ['-ig']
        _range_1 = ['és']
        _range_2 = ['között']
        self._span = re.compile(r"(?P<start>.+)(%s)\s+(?P<stop>.+)(%s)" %
                                ('|'.join(_span_1), '|'.join(_span_2)),
                                re.IGNORECASE)
        self._range = re.compile(r"(?P<start>.+)\s+(%s)\s+(?P<stop>.+)\s+(%s)"
                                 % ('|'.join(_range_1), '|'.join(_range_2)),
                                 re.IGNORECASE)


    def _get_int(self, val):
        """
        Convert the string to an integer if the value is not None. If the
        value is None, a zero is returned
        """
        if val is None:
            return 0
        else:
            return int(val.replace('.', ''))

#-------------------------------------------------------------------------
#
# Hungarian display
#
#-------------------------------------------------------------------------
class DateDisplayHU(DateDisplay):
    """
    Hungarian language date display class.
    """

    _bce_str = "i. e. %s"

    roman_months=(
                  "-.", "I.", "II.", "III.", "IV.", "V.", "VI.",
                  "VII.", "VIII.", "IX.", "X.", "XI.", "XII."
                 )

    formats = (
        "ÉÉÉÉ-HH-NN (ISO)",                              # 0
        "Alapértelmezett éééé. hh. nn.",                 # 1
        "Év hónap nap", # year, full month name, day     # 2
        "Év hó nap",    #year, short month name, day     # 3
        "Év római h.sz. nap"   #year, Roman number, day  # 4
        )
        # this definition must agree with its "_display_calendar" method

    display = DateDisplay.display_formatted

    def _display_calendar(self, date_val, long_months, short_months = None,
                          inflect=""):
        # this must agree with its locale-specific "formats" definition

        year = self._slash_year(date_val[2], date_val[3])

        if short_months is None:
            # Let the short formats work the same as long formats
            short_months = long_months

        if self.format == 0:
            return self.display_iso(date_val)

        elif self.format == 1:
            # Base defined Hungarian form
            if date_val[3]:
                return self.display_iso(date_val)
            else:
                if date_val[0]==0:          #No day
                    if date_val[1]==0:      #No month -> year
                        value="%s" % year
                    else:
                         value="%s. %02d." % (year, date_val[1])    #If no day -> year, month
                else:
                    value="%s. %02d. %02d." % (year, date_val[1], date_val[0])

        elif self.format == 2:
            # year, full month name, day

            if date_val[0]==0:
                if date_val[1]==0:
                    value="%s" % year
                else:
                     value="%s. %s" % (year, self.long_months[date_val[1]]) #If no day -> year, month
            else:
                 if date_val[1]==0:
                     value="%s. %s %02d." % (year, '-', date_val[0])   #To indicate somehow if the month is missing
                 else:
                      value="%s. %s %02d." % (year, self.long_months[date_val[1]], date_val[0])


        elif self.format == 3:
            #year, short month name, day

            if date_val[0]==0:
                if date_val[1]==0:
                    value="%s" % year
                else:
                    value="%s. %s" % (year, self.short_months[date_val[1]]) #If no day -> year, month
            else:
                 if date_val[1]==0:
                     value="%s. %s %02d." % (year, '-.', date_val[0])   #To indicate somehow if the month is missing
                 else:
                     value="%s. %s %02d." % (year, self.short_months[date_val[1]], date_val[0])


        elif self.format == 4:
            #year, Roman number, day

            if date_val[0]==0:
                if date_val[1]==0:
                    value="%s" % year
                else:
                     value="%s. %s" % (year, self.roman_months[date_val[1]]) #If no day -> year, month
            else:
                value="%s. %s %02d." % (year, self.roman_months[date_val[1]], date_val[0])


        else:
            # day month_name year
            value = self.dd_dformat04(date_val, inflect, long_months)

        if date_val[2] < 0:
            # TODO fix BUG 7064: non-Gregorian calendars wrongly use BCE notation for negative dates
            return self._bce_str % value
        else:
            return value

#-------------------------------------------------------------------------
#
# Register classes
#
#-------------------------------------------------------------------------
register_datehandler(
    ('hu_HU', 'hu', 'hungarian', 'Hungarian', 'magyar', ('%Y-%m-%d',)),
    DateParserHU, DateDisplayHU)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
# Copyright (C) 2013-2014  Fedir Zinchuk <fedikw[at]gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Ukrainian-specific classes for parsing and displaying dates.
"""

#-------------------------------------------------------------------------
#
# Python modules
#
#-------------------------------------------------------------------------
import re

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------

from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler

#-------------------------------------------------------------------------
#
# Ukrainian parser
#
#-------------------------------------------------------------------------
class DateParserUK(DateParser):
    """
    Convert a text string into a :class:`.Date` object. If the date cannot be
    converted, the text string is assigned.
    """

    # modifiers before the date
    modifier_to_int = {
        'перед'    : Date.MOD_BEFORE,
        'до'       : Date.MOD_BEFORE,
        'раніше'   : Date.MOD_BEFORE,

        'після'    : Date.MOD_AFTER,
        'п.'       : Date.MOD_AFTER,
        'за'       : Date.MOD_AFTER,

        'приблизно': Date.MOD_ABOUT,
        'прибл.'   : Date.MOD_ABOUT,
        'приб.'    : Date.MOD_ABOUT,
        'близько'  : Date.MOD_ABOUT,
        'бл.'      : Date.MOD_ABOUT,
        'біля'     : Date.MOD_ABOUT,
        }

    quality_to_int = {
        'за оцінкою'  : Date.QUAL_ESTIMATED,
        'за оц.'      : Date.QUAL_ESTIMATED,
        'оцінено'     : Date.QUAL_ESTIMATED,
        'орієнтовно'  : Date.QUAL_ESTIMATED,
        'приблизно'   : Date.QUAL_ESTIMATED,
        'прибл.'      : Date.QUAL_ESTIMATED,

        'підраховано' : Date.QUAL_CALCULATED,
        'підрах.'     : Date.QUAL_CALCULATED,
        'розраховано' : Date.QUAL_CALCULATED,
        'розрахунково' : Date.QUAL_CALCULATED,
        'розрах.'     : Date.QUAL_CALCULATED,
        }

    bce = [
        'до нашої ери', 'до н. е.', 'до н.е.',
        'до народження Христа'
        ] + DateParser.bce

    def init_strings(self):
        """
        This method compiles regular expression strings for matching dates.

        See DateParser.init_strings()
        """
        DateParser.init_strings(self)

        DateParser.hebrew_to_int.update({
            'мархешван': 2,
            'ве адар'  : 7,
            'адар бет' : 7,
            'авів'     : 8,
        })

        _span_1 = ['з', 'від']
        # b.c.e. pattern also have "до" so skip "до н."
        _span_2 = ['по', r'до(?!\s+н)']
        _range_1 = ['між']
        _range_2 = ['і', 'та']
        self._span = re.compile(r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)" %
                                ('|'.join(_span_1), '|'.join(_span_2)),
                                re.IGNORECASE)
        self._range = re.compile(r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
                                 % ('|'.join(_range_1), '|'.join(_range_2)),
                                 re.IGNORECASE)

#-------------------------------------------------------------------------
#
# Ukrainian displayer
#
#-------------------------------------------------------------------------
class DateDisplayUK(DateDisplay):
    """
    Ukrainian language date display class.
    """

    _bce_str = "%s до н.е."

    display = DateDisplay.display_formatted


#-------------------------------------------------------------------------
#
# Register classes
#
#-------------------------------------------------------------------------
register_datehandler(
    ('uk_UA', 'uk', 'ukrainian', 'Ukrainian', ('%d.%m.%Y',)),
    DateParserUK, DateDisplayUK)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2016 Tom Samstag
# Copyright (C) 2017 Serge Noiraud
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Unittest that tests place-specific filter rules
"""
import unittest
import os

from ....db.utils import import_as_dict
from ....filters import GenericFilterFactory
from ....const import DATA_DIR
from ....user import User

from ..place import (
    AllPlaces, HasCitation, HasGallery, HasIdOf, RegExpIdOf, HasNote,
    HasNoteRegexp, HasReferenceCountOf, HasSourceCount, HasSourceOf,
    PlacePrivate, MatchesSourceConfidence, HasData, HasNoLatOrLon,
    InLatLonNeighborhood, ChangedSince, HasTag, HasTitle, IsEnclosedBy,
    WithinArea
    )

TEST_DIR = os.path.abspath(os.path.join(DATA_DIR, "tests"))
EXAMPLE = os.path.join(TEST_DIR, "example.gramps")
GenericPlaceFilter = GenericFilterFactory('Place')

class BaseTest(unittest.TestCase):
    """
    Place rule tests.
    """

    @classmethod
    def setUpClass(cls):
        """
        Import example database.
        """
        cls.db = import_as_dict(EXAMPLE, User())

    def filter_with_rule(self, rule):
        """
        Apply a filter with the given rule.
        """
        filter_ = GenericPlaceFilter()
        filter_.add_rule(rule)
        results = filter_.apply(self.db)
        return set(results)

    def test_allplaces(self):
        """
        Test AllPlaces rule.
        """
        rule = AllPlaces([])
        self.assertEqual(len(self.filter_with_rule(rule)),
                         self.db.get_number_of_places())

    def test_hascitation(self):
        """
        Test HasCitation rule.
        """
        rule = HasCitation(['page 23', '', ''])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['YNUJQC8YM5EGRG868J']))

    def test_hasgallery(self):
        """
        Test HasGallery rule.
        """
        rule = HasGallery(['0', 'greater than'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['YNUJQC8YM5EGRG868J']))

    def test_hasidof(self):
        """
        Test HasIdOf rule.
        """
        rule = HasIdOf(['P0001'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['c96587262e91149933fcea5f20a']))

    def test_regexpidof(self):
        """
        Test RegExpIdOf rule.
        """
        rule = RegExpIdOf(['P000.'], use_regex=True)
        self.assertEqual(self.filter_with_rule(rule), set([
            'c96587262e91149933fcea5f20a', 'c96587262ff262aaac31f6db7af',
            'c96587262f24c33ab2420276737', 'c96587262e566596a225682bf53',
            'c9658726302661576894508202d', 'c96587262f8329d37b252e1b9e5',
            'c965872630664f33485fc18e75', 'c96587262fb7dbb954077cb1286',
            'c96587262f4a44183c65ff1e52', 'c96587262ed43fdb37bf04bdb7f',
            ]))

    def test_hasnote(self):
        """
        Test HasNote rule.
        """
        rule = HasNote([])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hasnoteregexp(self):
        """
        Test HasNoteRegexp rule.
        """
        rule = HasNoteRegexp(['.'], use_regex=True)
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hasreferencecountof(self):
        """
        Test HasReferenceCountOf rule.
        """
        rule = HasReferenceCountOf(['greater than', '35'])
        self.assertEqual(self.filter_with_rule(rule), set([
            'c96587262e566596a225682bf53', 'MATJQCJYH8ULRIRYTH',
            '5HTJQCSB91P69HY731', '4ECKQCWCLO5YIHXEXC',
            'c965872630a68ebd32322c4a30a']))

    def test_hassourcecount(self):
        """
        Test HasSourceCount rule.
        """
        rule = HasSourceCount(['1', 'equal to'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['YNUJQC8YM5EGRG868J']))

    def test_hassourceof(self):
        """
        Test HasSourceOf rule.
        """
        rule = HasSourceOf(['S0001'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['YNUJQC8YM5EGRG868J']))

    def test_placeprivate(self):
        """
        Test PlacePrivate rule.
        """
        rule = PlacePrivate([])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_matchessourceconfidence(self):
        """
        Test MatchesSourceConfidence rule.
        """
        rule = MatchesSourceConfidence(['2'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['YNUJQC8YM5EGRG868J']))

    def test_hasdata(self):
        """
        Test HasData rule.
        """
        rule = HasData(['Albany', 'County', ''])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['c9658726d602acadb74e330116a']))

    def test_hasnolatorlon(self):
        """
        Test HasNoLatOrLon rule.
        """
        rule = HasNoLatOrLon([])
        self.assertEqual(len(self.filter_with_rule(rule)), 921)

    def test_inlatlonneighborhood(self):
        """
        Test InLatLonNeighborhood rule.
        """
        rule = InLatLonNeighborhood(['30N', '90W', '2', '2'])
        self.assertEqual(self.filter_with_rule(rule), set([
            'C6WJQC0GDYP3HZDPR3', 'N88LQCRB363ES5WJ83',
            '03EKQCC2KTNLHFLDRJ', 'M9VKQCJV91X0M12J8']))

    def test_changedsince(self):
        """
        Test ChangedSince rule.
        """
        rule = ChangedSince(['2013-01-01', '2014-01-01'])
        self.assertEqual(len(self.filter_with_rule(rule)), 419)

    def test_hastag(self):
        """
        Test HasTag rule.
        """
        rule = HasTag(['ToDo'])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hastitle(self):
        """
        Test HasTitle rule.
        """
        rule = HasTitle(['Albany'])
        self.assertEqual(self.filter_with_rule(rule), set([
            '51VJQCXUP61H9JRL66', '9XBKQCE1LZ7PMJE56G',
            'c9658726d602acadb74e330116a', 'P9MKQCT08Z3YBJV5UB']))

    def test_isenclosedby(self):
        """
        Test IsEnclosedBy rule.
        """
        rule = IsEnclosedBy(['P0001', '0'])
        self.assertEqual(self.filter_with_rule(rule), set([
            'EAFKQCR0ED5QWL87EO', 'S22LQCLUZM135LVKRL', 'VDUJQCFP24ZV3O4ID2',
            'V6ALQCZZFN996CO4D', 'OC6LQCXMKP6NUVYQD8', 'CUUKQC6BY5LAZXLXC6',
            'PTFKQCKPHO2VC5SYKS', 'PHUJQCJ9R4XQO5Y0WS']))

    def test_withinarea(self):
        """
        Test within area rule.
        """
        rule = WithinArea(['P1339', 100, 0])
        self.assertEqual(self.filter_with_rule(rule), set([
            'KJUJQCY580EB77WIVO', 'TLVJQC4FD2CD9OYAXU', 'TE4KQCL9FDYA4PB6VW',
            'W9GLQCSRJIQ9N2TGDF']))

    def test_isenclosedby_inclusive(self):
        """
        Test IsEnclosedBy rule with inclusive option.
        """
        rule = IsEnclosedBy(['P0001', '1'])
        self.assertEqual(self.filter_with_rule(rule), set([
            'c96587262e91149933fcea5f20a', 'EAFKQCR0ED5QWL87EO',
            'S22LQCLUZM135LVKRL', 'VDUJQCFP24ZV3O4ID2', 'V6ALQCZZFN996CO4D',
            'OC6LQCXMKP6NUVYQD8', 'CUUKQC6BY5LAZXLXC6', 'PTFKQCKPHO2VC5SYKS',
            'PHUJQCJ9R4XQO5Y0WS']))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Russian-specific classes for parsing and displaying dates.
"""

#-------------------------------------------------------------------------
#
# Python modules
#
#-------------------------------------------------------------------------
import re

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler

#-------------------------------------------------------------------------
#
# Russian parser
#
#-------------------------------------------------------------------------
class DateParserRU(DateParser):
    modifier_to_int = {
        'перед'    : Date.MOD_BEFORE,
        'по'    : Date.MOD_BEFORE,
        'до'    : Date.MOD_BEFORE,
        'после' : Date.MOD_AFTER,
        'п.'    : Date.MOD_AFTER,
        'п'    : Date.MOD_AFTER,
        'с'     : Date.MOD_AFTER,
        'ок' : Date.MOD_ABOUT,
        'ок.'   : Date.MOD_ABOUT,
        'около'    : Date.MOD_ABOUT,
        'примерно'  : Date.MOD_ABOUT,
        'прим'     : Date.MOD_ABOUT,
        'прим.'     : Date.MOD_ABOUT,
        'приблизительно'  : Date.MOD_ABOUT,
        'приб.'  : Date.MOD_ABOUT,
        'прибл.'  : Date.MOD_ABOUT,
        'приб'  : Date.MOD_ABOUT,
        'прибл'  : Date.MOD_ABOUT,
        }


    quality_to_int = {
        'оценено'  : Date.QUAL_ESTIMATED,
        'оцен.'       : Date.QUAL_ESTIMATED,
        'оц.'        : Date.QUAL_ESTIMATED,
        'оцен'       : Date.QUAL_ESTIMATED,
        'оц'        : Date.QUAL_ESTIMATED,
        'вычислено'      : Date.QUAL_CALCULATED,
        'вычисл.'       : Date.QUAL_CALCULATED,
        'выч.' : Date.QUAL_CALCULATED,
        'вычисл'       : Date.QUAL_CALCULATED,
        'выч' : Date.QUAL_CALCULATED,
        }

    bce = [
        'до нашей эры', 'до н. э.', 'до н.э.',
        'до н э', 'до нэ'] + DateParser.bce

    def init_strings(self):
        DateParser.init_strings(self)
        DateParser.calendar_to_int.update({
            'персидский'    : Date.CAL_PERSIAN,
            'п'             : Date.CAL_PERSIAN,
        })
        _span_1 = ['с', 'от']
        #_span_2 = ['по', 'до'] # <-- clashes with bce parsing :-(
        _span_2 = ['по']
        _range_1 = ['между', r'меж\.', 'меж']
        _range_2 = ['и']
        self._span = re.compile(r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
                                % ('|'.join(_span_1), '|'.join(_span_2)),
                                re.IGNORECASE)
        self._range = re.compile(r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
                                 % ('|'.join(_range_1), '|'.join(_range_2)),
                                 re.IGNORECASE)

#-------------------------------------------------------------------------
#
# Russian displayer
#
#-------------------------------------------------------------------------
class DateDisplayRU(DateDisplay):
    """
    Russian language date display class.
    """

    # TODO fix BUG 7064: non-Gregorian calendars wrongly use BCE notation for negative dates
    # not refactoring _bce_str into base class because it'll be gone under #7064
    _bce_str = "%s до н.э."

    display = DateDisplay.display_formatted

    def dd_dformat04(self, date_val, inflect, long_months):
        """
        day month_name year -- for Russian locale
        """
        year = self._slash_year(date_val[2], date_val[3])
        if date_val[0] == 0:
            if date_val[1] == 0:
                return year
            else:
                return self.format_long_month_year(date_val[1], year,
                                                   inflect, long_months)
        elif date_val[1] == 0: # month is zero but day is not (see 8477)
            return self.display_iso(date_val)
        elif not hasattr(long_months[date_val[1]], 'f'): # not a Lexeme
            return "{day:d} {long_month} {year}".format(
                     day = date_val[0],
                     long_month = long_months[date_val[1]],
                     year = year)
        else:
            return "{day:d} {long_month.f[Р]} {year}".format(
                     day = date_val[0],
                     long_month = long_months[date_val[1]],
                     year = year)

    def dd_dformat05(self, date_val, inflect, short_months):
        """
        day month_abbreviation year -- for Russian locale
        """
        year = self._slash_year(date_val[2], date_val[3])
        if date_val[0] == 0:
            if date_val[1] == 0:
                return year
            else:
                return self.format_short_month_year(date_val[1], year,
                                                    inflect, short_months)
        elif date_val[1] == 0: # month is zero but day is not (see 8477)
            return self.display_iso(date_val)
        elif not hasattr(short_months[date_val[1]], 'f'): # not a Lexeme
            return "{day:d} {short_month} {year}".format(
                     day = date_val[0],
                     short_month = short_months[date_val[1]],
                     year = year)
        else:
            return "{day:d} {short_month.f[Р]} {year}".format(
                     day = date_val[0],
                     short_month = short_months[date_val[1]],
                     year = year)

#-------------------------------------------------------------------------
#
# Register classes
#
#-------------------------------------------------------------------------
register_datehandler(
    ('ru_RU', 'ru', 'russian', 'Russian', ('%d.%m.%Y',)),
    DateParserRU, DateDisplayRU)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2016 Tom Samstag
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Unittest that tests media-specific filter rules
"""
import unittest
import os

from ....db.utils import import_as_dict
from ....filters import GenericFilterFactory
from ....const import DATA_DIR
from ....user import User

from ..media import (
    AllMedia, HasIdOf, RegExpIdOf, HasCitation, HasNoteRegexp,
    HasNoteMatchingSubstringOf, HasReferenceCountOf, HasSourceCount,
    HasSourceOf, MediaPrivate, MatchesSourceConfidence, HasMedia,
    HasAttribute, ChangedSince, HasTag)

TEST_DIR = os.path.abspath(os.path.join(DATA_DIR, "tests"))
EXAMPLE = os.path.join(TEST_DIR, "example.gramps")
GenericMediaFilter = GenericFilterFactory('Media')

class BaseTest(unittest.TestCase):
    """
    Media rule tests.
    """

    @classmethod
    def setUpClass(cls):
        """
        Import example database.
        """
        cls.db = import_as_dict(EXAMPLE, User())

    def filter_with_rule(self, rule):
        """
        Apply a filter with the given rule.
        """
        filter_ = GenericMediaFilter()
        filter_.add_rule(rule)
        results = filter_.apply(self.db)
        return set(results)

    def test_allmedia(self):
        """
        Test AllMedia rule.
        """
        rule = AllMedia([])
        self.assertEqual(len(self.filter_with_rule(rule)),
                         self.db.get_number_of_media())

    def test_hasidof(self):
        """
        Test HasIdOf rule.
        """
        rule = HasIdOf(['O0000'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['b39fe1cfc1305ac4a21']))

    def test_regexpidof(self):
        """
        Test RegExpIdOf rule.
        """
        rule = RegExpIdOf(['O000.'], use_regex=True)
        self.assertEqual(self.filter_with_rule(rule), set([
            'F0QIGQFT275JFJ75E8', '78V2GQX2FKNSYQ3OHE',
            'b39fe1cfc1305ac4a21', 'F8JYGQFL2PKLSYH79X',
            'B1AUFQV7H8R9NR4SZM']))

    def test_hascitation(self):
        """
        Test HasCitation rule.
        """
        rule = HasCitation(['page 21', '', ''])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['B1AUFQV7H8R9NR4SZM']))

    def test_hasnoteregexp(self):
        """
        Test HasNoteRegexp rule.
        """
        rule = HasNoteRegexp(['.'], use_regex=True)
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hasnotematchingsubstringof(self):
        """
        Test HasNoteMatchingSubstringOf rule.
        """
        rule = HasNoteMatchingSubstringOf(['some text'])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hasreferencecountof(self):
        """
        Test HasReferenceCountOf rule.
        """
        rule = HasReferenceCountOf(['greater than', '1'])
        self.assertEqual(self.filter_with_rule(rule), set([
            '238CGQ939HG18SS5MG', 'b39fe1cfc1305ac4a21']))

    def test_hassourcecount(self):
        """
        Test HasSourceCount rule.
        """
        rule = HasSourceCount(['1', 'equal to'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['B1AUFQV7H8R9NR4SZM']))

    def test_hassourceof(self):
        """
        Test HasSourceOf rule.
        """
        rule = HasSourceOf(['S0001'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['B1AUFQV7H8R9NR4SZM']))

    def test_mediaprivate(self):
        """
        Test MediaPrivate rule.
        """
        rule = MediaPrivate([])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_matchessourceconfidence(self):
        """
        Test MatchesSourceConfidence rule.
        """
        rule = MatchesSourceConfidence(['2'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['B1AUFQV7H8R9NR4SZM']))

    def test_hasmedia(self):
        """
        Test HasMedia rule.
        """
        rule = HasMedia(['mannschaft', 'image/jpeg', '.jpg', '1897'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['238CGQ939HG18SS5MG']))

    def test_hasattribute(self):
        """
        Test HasAttribute rule.
        """
        rule = HasAttribute(['Description', ''])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['B1AUFQV7H8R9NR4SZM']))

    def test_changedsince(self):
        """
        Test ChangedSince rule.
        """
        rule = ChangedSince(['2010-01-01', '2016-01-01'])
        self.assertEqual(self.filter_with_rule(rule), set([
            'b39fe1cfc1305ac4a21', 'B1AUFQV7H8R9NR4SZM',
            '238CGQ939HG18SS5MG', 'F0QIGQFT275JFJ75E8']))

    def test_hastag(self):
        """
        Test HasTag rule.
        """
        rule = HasTag(['ToDo'])
        self.assertEqual(self.filter_with_rule(rule),
                         set(['238CGQ939HG18SS5MG']))


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python3
""" Test program for import modules
"""
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (c) 2016 Gramps Development Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#

import unittest
import os
import sys
import re
import locale
from time import localtime, strptime
from unittest.mock import patch
#import logging

from gramps.gen.utils.config import config
config.set('preferences.date-format', 0)
from gramps.gen.db.utils import import_as_dict
from gramps.gen.merge.diff import diff_dbs, to_struct
from gramps.gen.simple import SimpleAccess
from gramps.gen.utils.id import set_det_id
from gramps.gen.user import User
from gramps.gen.const import TEMP_DIR, DATA_DIR
from gramps.test.test_util import capture
from gramps.plugins.export.exportxml import XmlWriter

#logger = logging.getLogger(__name__)

# the following defines where to find the test import and result files
TEST_DIR = os.path.abspath(os.path.join(DATA_DIR, "tests"))

# ------------------------------------------------------------------
#  Local Functions
# ------------------------------------------------------------------

# These tests assume a US date and time format.
try:
    locale.setlocale(locale.LC_ALL, 'en_US.utf8')
except locale.Error:     # seems to fail on Windows system for some reason
    locale.setlocale(locale.LC_ALL, 'English_United States')


def mock_time(*args):
    """
    Mock up a dummy to replace the varying 'time string results'
    """
    return 946101600.

def mock_localtime(*args):
    """
    Mock up a dummy to replace the varying 'time string results'
    """
    return strptime("25 Dec 1999", "%d %b %Y")

class TestImports(unittest.TestCase):
    """The test class cases will be dynamically created at import time from
    files to be tested.  The following defs are used by the test cases
    """

    def prepare_result(self, diffs, added, missing):
        """ Looks through the diffs, added, and missing items and begins
        reporting process.  Returns True if there were significant errors.
        """
        # pylint: disable=E1101
        # pylint does not like dynamically created methods
        deltas = False
        if diffs:
            for diff in diffs:
                obj_type, item1, item2 = diff
                msg = self._report_diff(obj_type, to_struct(item1),
                                        to_struct(item2))
                if msg != "":
                    if hasattr(item1, "gramps_id"):
                        self.msg += "%s: %s  handle=%s\n" % \
                            (obj_type, item1.gramps_id,
                             getattr(item1, "handle"))
                    else:
                        self.msg += "%s: %s\n" % (obj_type, item1.get_name())
                    self.msg += msg
                    deltas = True
        if missing:
            deltas = True
            sac = SimpleAccess(self.database1)
            for pair in missing:
                obj_type, item = pair
                self.msg += "Missing %s: %s\n" % (obj_type, sac.describe(item))
        if added:
            deltas = True
            sac = SimpleAccess(self.database2)
            for pair in added:
                obj_type, item = pair
                self.msg += "Added %s: %s\n" % (obj_type, sac.describe(item))
        return deltas


    def _report_diff(self, path, struct1, struct2):
        """
        Compare two struct objects and report differences.
        """
        msg = ""
        if struct1 == struct2:
            return ""
        elif (isinstance(struct1, (list, tuple)) or
              isinstance(struct2, (list, tuple))):
            len1 = len(struct1) if isinstance(struct1, (list, tuple)) else 0
            len2 = len(struct2) if isinstance(struct2, (list, tuple)) else 0
            for pos in range(max(len1, len2)):
                value1 = struct1[pos] if pos < len1 else None
                value2 = struct2[pos] if pos < len2 else None
                msg += self._report_diff(path + ("[%d]" % pos), value1, value2)
        elif isinstance(struct1, dict) or isinstance(struct2, dict):
            keys = struct1.keys() if isinstance(struct1, dict)\
                                  else struct2.keys()
            for key in keys:
                value1 = struct1[key] if struct1 is not None else None
                value2 = struct2[key] if struct2 is not None else None
                if key == "dict":  # a raw dict, not a struct
                    msg += _report_details(path, value1, value2)
                else:
                    msg += self._report_diff(path + "." + key, value1, value2)
        else:
            msg += _report_details(path, struct1, struct2)
        return msg


def _report_details(path, diff1, diff2):
    """ Checks if a detail is significant, needs adjusting for xml filter
    effects, and returns a string describing the specific difference.
    """
    if isinstance(diff1, bool):
        desc1 = repr(diff1)
    else:
        desc1 = str(diff1) if diff1 else ""
    if isinstance(diff2, bool):
        desc2 = repr(diff2)
    else:
        desc2 = str(diff2) if diff2 else ""
    d1t = type(diff1)
    d2t = type(diff2)
    # the xml exporter edits the data base by stripping spaces, so
    # we have to ignore these differences
    if d1t == str:
        diff1 = diff1.strip()
    if d2t == str:
        diff2 = diff2.strip()
    d1l = len(diff1) if d1t == str else ""
    d2l = len(diff2) if d2t == str else ""
    # 'change' date is not significant for comparison
    if path.endswith(".change"):
        return ""
    # the xml exporter edits the data base by converting media path
    # to unix '/' conventions, so we have to ignore these differences
    if path == "Media.path":
        diff1 = diff1.replace('\\', '/')
    if diff1 != diff2:
        msg = "  Diff on: %s\n    %s%s: %s\n    %s%s: %s\n" % \
            (_format_struct_path(path), d1t, d1l, desc1,
             d2t, d2l, desc2)
        return msg
    return ""


def _format_struct_path(path):
    """ prepares a 'path' string for the report out of the structure.
    """
    retval = ""
    parts = path.split(".")
    for part in parts:
        if retval:
            retval += ", "
        if "[" in part and "]" in part:
            part, index = re.match(r"(.*)\[(\d*)\]", part).groups()
            retval += "%s #%s" % (part.replace("_", " "), int(index) + 1)
        else:
            retval += part
    return retval


def make_tst_function(tstfile, file_name):
    """ This is here to support the dynamic function creation.  This creates
    the test function (a method, to be precise).
    """

    @patch('gramps.plugins.db.dbapi.dbapi.time')
    @patch('gramps.plugins.db.bsddb.write.time')
    @patch('gramps.gen.utils.unknown.localtime')
    @patch('gramps.gen.utils.unknown.time')
    @patch('time.localtime')
    def tst(self, mockptime, mocktime, mockltime, mockwtime, mockdtime):
        """ This compares the import file with the expected result '.gramps'
        file.
        """
        mockptime.side_effect = mock_localtime
        mocktime.side_effect = mock_time
        mockltime.side_effect = mock_localtime
        mockwtime.side_effect = mock_time
        mockdtime.side_effect = mock_time
        fn1 = os.path.join(TEST_DIR, tstfile)
        fn2 = os.path.join(TEST_DIR, (file_name + ".gramps"))
        fres = os.path.join(TEMP_DIR, (file_name + ".difs"))
        fout = os.path.join(TEMP_DIR, (file_name + ".gramps"))
        if "_dfs" in tstfile:
            config.set('preferences.default-source', True)
            config.set('preferences.tag-on-import-format', "Imported")
            config.set('preferences.tag-on-import', True)
            skp_imp_adds = False
        else:
            skp_imp_adds = True
            config.set('preferences.default-source', False)
            config.set('preferences.tag-on-import', False)
        try:
            os.remove(fres)
            os.remove(fout)
        except OSError:
            pass
        #logger.info("\n**** %s ****", tstfile)
        set_det_id(True)
        with capture(None) as output:
            self.user = User()
            self.database1 = import_as_dict(fn1, self.user,
                                            skp_imp_adds=skp_imp_adds)
            set_det_id(True)
            self.database2 = import_as_dict(fn2, self.user)
        self.assertIsNotNone(self.database1,
                             "Unable to import file: %s" % fn1)
        self.assertIsNotNone(self.database2,
                             "Unable to import expected result file: %s" % fn2)
        if self.database2 is None or self.database1 is None:
            return
        diffs, added, missing = diff_dbs(self.database1,
                                         self.database2, self.user)
        self.msg = "Mismatch on file: %s\n" % tstfile
        deltas = self.prepare_result(diffs, added, missing)
        # We save a copy of any issues in the users Gramps TEMP_DIR in a file
        # with a '.difs' extension, as well as usual unittest report
        # Also we save the .gramps file from the import so user can perform
        # Diff himself for better context.
        if deltas:
            writer = XmlWriter(self.database1, self.user,
                               strip_photos=0, compress=0)
            writer.write(fout)
            hres = open(fres, mode='w', encoding='utf-8',
                        errors='replace')
            hres.write(self.msg)
            hres.close()
            # let's see if we have any allowed exception file
            fdif = os.path.join(TEST_DIR, (file_name + ".difs"))
            try:
                hdif = open(fdif)
                msg = hdif.read()
                hdif.close()
            except (FileNotFoundError, IOError):
                msg = ""
            # if exception file matches exactly, we are done.
            if self.msg != msg:
                self.msg = "\n****Captured Output****\n" + output[0] + \
                    "\n****Captured Err****\n" + output[1] + \
                    "\n****End Capture Err****\n" + self.msg
                self.fail(self.msg)
    return tst

# let's see if we have a single file to run, example;
#    "python test_import.py -i sample.ged"
# This only works for files in normal test directory, so don't add a path
#pylint: disable=invalid-name
_tstfile = ""
if __name__ == "__main__":
    for i, option in enumerate(sys.argv):
        if option == '-i':
            _tstfile = sys.argv[i+1]
            del sys.argv[i]
            del sys.argv[i]
# The following code dynamically creates the methods for each test file.
# The methods are inserted at load time into the 'TestImports' class
# via the modules' globals, taking advantage that they are a dict.
if _tstfile:                             # single file mode
    (fname, ext) = os.path.splitext(os.path.basename(_tstfile))
    test_func = make_tst_function(_tstfile, fname)
    tname = "test_" + _tstfile.replace("-", "_").replace(".", "_")
    test_func.__name__ = tname
    test_func.__doc__ = tname
    setattr(TestImports, tname, test_func)
else:
    for _tstfile in os.listdir(TEST_DIR):
        (fname, ext) = os.path.splitext(os.path.basename(_tstfile))
        if _tstfile != "SAMPLE.DEF" and (ext in (".gramps", ".difs", ".bak") \
                or not fname.startswith("imp_")):
            continue
        test_func = make_tst_function(_tstfile, fname)
        tname = "test_" + _tstfile.replace("-", "_").replace(".", "_")
        test_func.__name__ = tname
        test_func.__doc__ = tname
        setattr(TestImports, tname, test_func)
    test_func = None  # keep nosetest from finding last one again

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2003-2005  Donald N. Allingham
# Copyright (C) 2008       Brian G. Matherly
# Copyright (C) 2010       Andrew I Baznikin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# Written by Alex Roitman, largely based on relationship.py by Don Allingham
# and on valuable input from Frode Jemtland
"""
Norwegian-Specific classes for relationships.
"""

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------

from gramps.gen.lib import Person
import gramps.gen.relationship

#-------------------------------------------------------------------------

_cousin_level = [ "", "", #brother/sister, fetter/kusine -- these are taken care of separately
"tremenning", "firemenning", "femmenning",
"seksmenning", "sjumenning", "åttemenning",
"nimenning", "timenning", "ellevemenning",
"tolvmenning", "tretenmenning", "fjortenmenning",
"femtenmenning", "sekstenmenning", "syttenmenning",
"attenmenning", "nittenmenning", "tjuemenning" ]

_cousin_terms = _cousin_level + ["fetter", "kusine"]
#-------------------------------------------------------------------------
#
#
#
#-------------------------------------------------------------------------
class RelationshipCalculator(gramps.gen.relationship.RelationshipCalculator):
    """
    RelationshipCalculator Class
    """

    def __init__(self):
        gramps.gen.relationship.RelationshipCalculator.__init__(self)

    def get_parents(self, level):
        if level == 0:
            return "forelder"
        else:
            return "ane i %d-te generationen" % (level+1)

    def get_cousin(self, level):
        if level > len(_cousin_level)-1:
            # FIXME: use Norwegian term term here,
            # UPDATED by Frode: unsure about the expretion "en fjern slektning", should it be just "fjern slektning".
            # Need to see it used in the program to get the understanding.
            return "en fjern slektning"
        else:
            return _cousin_level[level]

    def pair_up(self, rel_list):
        result = []
        item = ""
        for word in rel_list[:]:
            if not word:
                continue
            if word in _cousin_terms:
                if item:
                    result.append(item)
                    item = ""
                result.append(word)
                continue
            if item:
                if word == 'sønne':
                    word = 'sønn'
                result.append(item + word)
                item = ""
            else:
                item = word
        if item:
            result.append(item)
        gen_result = [ item + 's' for item in result[0:-1] ]
        return ' '.join(gen_result+result[-1:])

    def get_direct_ancestor(self, person, rel_string):
        result = []
        for ix in range(len(rel_string)):
            if rel_string[ix] == 'f':
                result.append('far')
            else:
                result.append('mor')
        return self.pair_up(result)

    def get_direct_descendant(self, person, rel_string):
        result = []
        for ix in range(len(rel_string)-2, -1, -1):
            if rel_string[ix] == 'f':
                result.append('sønne')
            else:
                result.append('datter')
        if person == Person.MALE:
            result.append('sønn')
        else:
            result.append('datter')
        return self.pair_up(result)

    def get_ancestors_cousin(self, person, rel_string_long, rel_string_short):
        result = []
        removed = len(rel_string_long)-len(rel_string_short)
        level = len(rel_string_short)-1
        for ix in range(removed):
            if rel_string_long[ix] == 'f':
                result.append('far')
            else:
                result.append('mor')
        if level > 1:
            result.append(self.get_cousin(level))
        elif person == Person.MALE:
            result.append('fetter')
        else:
            result.append('kusine')
        main_string = self.get_two_way_rel(person, rel_string_short, rel_string_long)
        aux_string = self.pair_up(result)
        return "%s (%s)" % (main_string, aux_string)

    def get_cousins_descendant(self, person, rel_string_long, rel_string_short):
        result = []
        aux_string = ""
        removed = len(rel_string_long)-len(rel_string_short)-1
        level = len(rel_string_short)-1
        if level > 1:       # Cousin terms without gender
            result.append(self.get_cousin(level))
        elif level == 1:    # gender-dependent fetter/kusine
            if rel_string_long[removed] == 'f':
                result.append('fetter')
            else:
                result.append('kusine')
        elif rel_string_long[removed] == 'f':
            result.append('bror')
        else:
            result.append('søster')
        for ix in range(removed-1, -1, -1):
            if rel_string_long[ix] == 'f':
                result.append('sønn')
            else:
                result.append('datter')
        if person == Person.MALE:
            result.append('sønn')
        else:
            result.append('datter')
        main_string = self.get_two_way_rel(person, rel_string_long, rel_string_short)
        if level:
            aux_string = " (%s)" % self.pair_up(result)
        return "%s%s" % (main_string, aux_string)

    def get_ancestors_brother(self, rel_string):
        result = []
        for ix in range(len(rel_string)-1):
            if rel_string[ix] == 'f':
                result.append('far')
            else:
                result.append('mor')
        result.append('bror')
        return self.pair_up(result)

    def get_ancestors_sister(self, rel_string):
        result = []
        for ix in range(len(rel_string)-1):
            if rel_string[ix] == 'f':
                result.append('far')
            else:
                result.append('mor')
        result.append('søster')
        return self.pair_up(result)

    def get_two_way_rel(self, person, first_rel_string, second_rel_string):
        result = []
        for ix in range(len(second_rel_string)-1):
            if second_rel_string[ix] == 'f':
                result.append('far')
            else:
                result.append('mor')
        if len(first_rel_string)>1:
            if first_rel_string[-2] == 'f':
                result.append('bror')
            else:
                result.append('søster')
            for ix in range(len(first_rel_string)-3, -1, -1):
                if first_rel_string[ix] == 'f':
                    result.append('sønne')
                else:
                    result.append('datter')
            if person == Person.MALE:
                result.append('sønn')
            else:
                result.append('datter')
        else:
            if person == Person.MALE:
                result.append('bror')
            else:
                result.append('søster')
        return self.pair_up(result)

    def get_relationship(self,
                        secondRel, firstRel, orig_person, other_person):

        common = ""
        if not firstRel:
            if not secondRel:
                return ('', common)
            else:
                return (self.get_direct_ancestor(other_person, secondRel), common)
        elif not secondRel:
            return (self.get_direct_descendant(other_person, firstRel), common)
        elif len(firstRel) == 1:
            if other_person == Person.MALE:
                return (self.get_ancestors_brother(secondRel), common)
            else:
                return (self.get_ancestors_sister(secondRel), common)
        elif len(secondRel) >= len(firstRel):
            return (self.get_ancestors_cousin(other_person, secondRel, firstRel), common)
        else:
            return (self.get_cousins_descendant(other_person, firstRel, secondRel), common)

    def get_single_relationship_string(self, Ga, Gb, gender_a, gender_b,
                                    reltocommon_a, reltocommon_b,
                                    only_birth=True,
                                    in_law_a=False, in_law_b=False):
        return self.get_relationship(reltocommon_a, reltocommon_b, gender_a, gender_b)[0];

    def get_sibling_relationship_string(self, sib_type, gender_a, gender_b,
                                        in_law_a=False, in_law_b=False):
        return self.get_two_way_rel(gender_b, "", "")

if __name__ == "__main__":

    # Test function. Call it as follows from the command line (so as to find
    #        imported modules):
    #    export PYTHONPATH=/path/to/gramps/src
    # python src/plugins/rel/rel_no.py
    # (Above not needed here)

    """TRANSLATORS, copy this if statement at the bottom of your
        rel_xx.py module, and test your work with:
        python src/plugins/rel/rel_xx.py
    """
    from gramps.gen.relationship import test
    RC = RelationshipCalculator()
    test(RC, True)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
# Copyright (C) 2009       Benny Malengier
# Copyright (C) 2009-2010  Stephen George
# Copyright (C) 2010       Doug Blank <doug.blank@gmail.com>
# Copyright (C) 2011       Paul Franklin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#-------------------------------------------------------------------------
#
# Python modules
#
#-------------------------------------------------------------------------
import sys
import os
import signal

import logging

LOG = logging.getLogger(".")

from subprocess import Popen, PIPE

#-------------------------------------------------------------------------
# process 'safe mode'; set up for a temp directory for user data
# actual directory paths set up in const module
if "-S" in sys.argv or "--safe" in sys.argv:
    from tempfile import TemporaryDirectory
    tempdir = TemporaryDirectory(prefix='gramps_')
    os.environ['SAFEMODE'] = tempdir.name

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
from .gen.const import APP_GRAMPS, USER_DIRLIST, HOME_DIR, ORIG_HOME_DIR
from .gen.constfunc import mac
from .version import VERSION_TUPLE
from .gen.constfunc import win, get_env_var
from .gen.config import config
from .gen.errors import HandleError

#-------------------------------------------------------------------------
#
# Instantiate Localization
#
#-------------------------------------------------------------------------

from .gen.const import GRAMPS_LOCALE as glocale
_ = glocale.translation.gettext

#-------------------------------------------------------------------------
#
# Ensure that output is encoded correctly to stdout and
# stderr. This is much less cumbersome and error-prone than
# encoding individual outputs:
#
#-------------------------------------------------------------------------

try:
    # On Darwin sys.getdefaultencoding() is correct, on Win32 it's
    # sys.stdout.enoding, and on Linux they're both right.
    if mac():
        _encoding = sys.getdefaultencoding()
    else:
        _encoding = sys.stdout.encoding
except:
    _encoding = "UTF-8"
try:
    sys.stdout = open(sys.stdout.fileno(), mode='w', encoding=_encoding,
                  buffering=1, errors='backslashreplace')
    sys.stderr = open(sys.stderr.fileno(), mode='w', encoding=_encoding,
                  buffering=1, errors='backslashreplace')
except:
    pass

#-------------------------------------------------------------------------
#
# Setup logging
#
# Ideally, this needs to be done before any Gramps modules are
# imported, so that any code that is executed as the modules are
# imported can log errors or warnings.  const and constfunc have to be
# imported before this code is executed because they are used in this
# code. That unfortunately initializes GrampsLocale, so it has its own
# logging setup during initialization.
#-------------------------------------------------------------------------
"""Setup basic logging support."""

# Setup a formatter
form = logging.Formatter(fmt="%(asctime)s.%(msecs).03d: %(levelname)s: "
                             "%(filename)s: line %(lineno)d: %(message)s",
                         datefmt='%Y-%m-%d %H:%M:%S')

# Create the log handlers
if win():
    # If running in GUI mode redirect stdout and stderr to log file
    if not sys.stdout:
        logfile = os.path.join(HOME_DIR,
            "Gramps%s%s.log") % (VERSION_TUPLE[0],
            VERSION_TUPLE[1])
        # We now carry out the first step in build_user_paths(), to make sure
        # that the user home directory is available to store the log file. When
        # build_user_paths() is called, the call is protected by a try...except
        # block, and any failure will be logged. However, if the creation of the
        # user directory fails here, there is no way to report the failure,
        # because stdout/stderr are not available, and neither is the logfile.
        if os.path.islink(HOME_DIR):
            pass  # ok
        elif not os.path.isdir(HOME_DIR):
            os.makedirs(HOME_DIR)
        sys.stdout = sys.stderr = open(logfile, "w", encoding='utf-8')
stderrh = logging.StreamHandler(sys.stderr)
stderrh.setFormatter(form)
stderrh.setLevel(logging.DEBUG)

# Setup the base level logger, this one gets
# everything.
l = logging.getLogger()
l.setLevel(logging.WARNING)
l.addHandler(stderrh)


def exc_hook(err_type, value, t_b):
    ''' put a hook on to catch any completely unhandled exceptions. '''
    if err_type == KeyboardInterrupt:
        # Ctrl-C is not a bug.
        return
    if err_type == IOError:
        # strange Windows logging error on close
        return
    if err_type == HandleError and 'not found' in value.value:
        # tell Gramps to run check & repair on next start
        config.set('behavior.runcheck', True)
        config.save()
    # Use this to show variables in each frame:
    #from gramps.gen.utils.debug import format_exception
    import traceback
    LOG.error("Unhandled exception\n" +
              "".join(traceback.format_exception(err_type, value, t_b)))

sys.excepthook = exc_hook

from .gen.mime import mime_type_is_defined


#-------------------------------------------------------------------------
#
# Minimum version check
#
#-------------------------------------------------------------------------

MIN_PYTHON_VERSION = (3, 3, 0, '', 0)
if not sys.version_info >= MIN_PYTHON_VERSION:
    logging.warning(_("Your Python version does not meet the "
             "requirements. At least python %(v1)d.%(v2)d.%(v3)d is needed to"
             " start Gramps.\n\n"
             "Gramps will terminate now.") % {
             'v1': MIN_PYTHON_VERSION[0],
             'v2': MIN_PYTHON_VERSION[1],
             'v3': MIN_PYTHON_VERSION[2]})
    sys.exit(1)

#-------------------------------------------------------------------------
#
# Gramps libraries
#
#-------------------------------------------------------------------------
try:
    signal.signal(signal.SIGCHLD, signal.SIG_DFL)
except:
    pass

args = sys.argv

def build_user_paths():
    """ check/make user-dirs on each Gramps session"""
    for path in USER_DIRLIST:
        if os.path.islink(path):
            pass  # ok
        elif not os.path.isdir(path):
            os.makedirs(path)

def show_settings():
    """
    Shows settings of all of the major components.
    """
    py_str = '%d.%d.%d' % sys.version_info[:3]
    try:
        import gi
        gi.require_version('Gtk', '3.0')
        from gi.repository import Gtk
        try:
            gtkver_str = '%d.%d.%d' % (Gtk.get_major_version(),
                        Gtk.get_minor_version(), Gtk.get_micro_version())
        except:  # any failure to 'get' the version
            gtkver_str = 'unknown version'
    except (ImportError, ValueError):
        gtkver_str = 'not found'
    # no DISPLAY is a RuntimeError in an older pygtk (e.g. 2.17 in Fedora 14)
    except RuntimeError:
        gtkver_str = 'DISPLAY not set'
    # exept TypeError: To handle back formatting on version split

    try:
        from gi.repository import GObject
        try:
            pygobjectver_str = '%d.%d.%d' % GObject.pygobject_version
        except:  # any failure to 'get' the version
            pygobjectver_str = 'unknown version'

    except ImportError:
        pygobjectver_str = 'not found'

    try:
        from gi.repository import Pango
        try:
            pangover_str = Pango.version_string()
        except:  # any failure to 'get' the version
            pangover_str = 'unknown version'

    except ImportError:
        pangover_str = 'not found'

    try:
        import cairo
        try:
            pycairover_str = '%d.%d.%d' % cairo.version_info
            cairover_str = cairo.cairo_version_string()
        except:  # any failure to 'get' the version
            pycairover_str = 'unknown version'
            cairover_str = 'unknown version'

    except ImportError:
        pycairover_str = 'not found'
        cairover_str = 'not found'

    try:
        from gi import Repository
        repository = Repository.get_default()
        if repository.enumerate_versions("OsmGpsMap"):
            import gi
            gi.require_version('OsmGpsMap', '1.0')
            from gi.repository import OsmGpsMap as osmgpsmap
            try:
                osmgpsmap_str = osmgpsmap._version
            except:  # any failure to 'get' the version
                osmgpsmap_str = 'unknown version'
        else:
            osmgpsmap_str = 'not found'

    except ImportError:
        osmgpsmap_str = 'not found'

    try:
        from gi import Repository
        repository = Repository.get_default()
        if repository.enumerate_versions("GExiv2"):
            import gi
            gi.require_version('GExiv2', '0.10')
            from gi.repository import GExiv2
            try:
                gexiv2_str = GExiv2._version
            except:  # any failure to 'get' the version
                gexiv2_str = 'unknown version'
        else:
            gexiv2_str = 'not found'

    except ImportError:
        gexiv2_str = 'not found'
    except ValueError:
        gexiv2_str = 'not new enough'

    try:
        import PyICU
        try:
            pyicu_str = PyICU.VERSION
            icu_str = PyICU.ICU_VERSION
        except:  # any failure to 'get' the version
            pyicu_str = 'unknown version'
            icu_str = 'unknown version'

    except ImportError:
        pyicu_str = 'not found'
        icu_str = 'not found'

    try:
        import bsddb3 as bsddb
        bsddb_str = bsddb.__version__
        bsddb_db_str = str(bsddb.db.version()).replace(', ', '.')\
                                        .replace('(', '').replace(')', '')
        bsddb_location_str = bsddb.__file__
    except:
        bsddb_str = 'not found'
        bsddb_db_str = 'not found'
        bsddb_location_str = 'not found'

    try:
        import sqlite3
        sqlite3_py_version_str = sqlite3.version
        sqlite3_version_str = sqlite3.sqlite_version
        sqlite3_location_str = sqlite3.__file__
    except:
        sqlite3_version_str = 'not found'
        sqlite3_py_version_str = 'not found'
        sqlite3_location_str = 'not found'

    try:
        from .gen.const import VERSION
        gramps_str = VERSION
    except:
        gramps_str = 'not found'

    if hasattr(os, "uname"):
        kernel = os.uname()[2]
    else:
        kernel = None

    lang_str = get_env_var('LANG', 'not set')
    language_str = get_env_var('LANGUAGE', 'not set')
    grampsi18n_str = get_env_var('GRAMPSI18N', 'not set')
    grampshome_str = get_env_var('GRAMPSHOME', 'not set')
    grampsdir_str = get_env_var('GRAMPSDIR', 'not set')
    gramps_resources_str = get_env_var('GRAMPS_RESOURCES', 'not set')

    try:
        dotversion_str = Popen(['dot', '-V'], stderr=PIPE).communicate(input=None)[1]
        if isinstance(dotversion_str, bytes) and sys.stdin.encoding:
            dotversion_str = dotversion_str.decode(sys.stdin.encoding)
        if dotversion_str:
            dotversion_str = dotversion_str.replace('\n', '')[23:27]
    except:
        dotversion_str = 'Graphviz not in system PATH'

    try:
        if win():
            try:
                gsversion_str = Popen(['gswin32c', '--version'], stdout=PIPE).communicate(input=None)[0]
            except:
                gsversion_str = Popen(['gswin64c', '--version'], stdout=PIPE).communicate(input=None)[0]
        else:
            gsversion_str = Popen(['gs', '--version'], stdout=PIPE).communicate(input=None)[0]
        if isinstance(gsversion_str, bytes) and sys.stdin.encoding:
            gsversion_str = gsversion_str.decode(sys.stdin.encoding)
        if gsversion_str:
            gsversion_str = gsversion_str.replace('\n', '')
    except:
        gsversion_str = 'Ghostscript not in system PATH'

    os_path = get_env_var('PATH', 'not set')
    os_path = os_path.split(os.pathsep)

    print("Gramps Settings:")
    print("----------------")
    print(' python    : %s' % py_str)
    print(' gramps    : %s' % gramps_str)
    print(' gtk++     : %s' % gtkver_str)
    print(' pygobject : %s' % pygobjectver_str)
    print(' pango     : %s' % pangover_str)
    print(' cairo     : %s' % cairover_str)
    print(' pycairo   : %s' % pycairover_str)
    print(' osmgpsmap : %s' % osmgpsmap_str)
    print(' GExiv2    : %s' % gexiv2_str)
    print(' ICU       : %s' % icu_str)
    print(' PyICU     : %s' % pyicu_str)
    print(' o.s.      : %s' % sys.platform)
    if kernel:
        print(' kernel    : %s' % kernel)
    print('')
    print("Environment settings:")
    print("---------------------")
    print(' LANG      : %s' % lang_str)
    print(' LANGUAGE  : %s' % language_str)
    print(' GRAMPSI18N: %s' % grampsi18n_str)
    print(' GRAMPSHOME: %s' % grampshome_str)
    print(' GRAMPSDIR : %s' % grampsdir_str)
    if __debug__:
        print(' GRAMPS_RESOURCES : %s' % gramps_resources_str)
    print(' PYTHONPATH:')
    for folder in sys.path:
        print("   ", folder)
    print('')
    print("Non-python dependencies:")
    print("------------------------")
    print(' Graphviz  : %s' % dotversion_str)
    print(' Ghostscr. : %s' % gsversion_str)
    print('')
    print("System PATH env variable:")
    print("-------------------------")
    for folder in os_path:
        print("    ", folder)
    print('')
    print("Databases:")
    print("-------------------------")
    print(' bsddb     :')
    print('     version     : %s' % bsddb_str)
    print('     db version  : %s' % bsddb_db_str)
    print('     location    : %s' % bsddb_location_str)
    print(' sqlite3   :')
    print('     version     : %s' % sqlite3_version_str)
    print('     py version  : %s' % sqlite3_py_version_str)
    print('     location    : %s' % sqlite3_location_str)
    print('')

def run():
    error = []

    try:
        build_user_paths()
    except OSError as msg:
        error += [(_("Configuration error:"), str(msg))]
        return error
    except msg:
        LOG.error("Error reading configuration.", exc_info=True)
        return [(_("Error reading configuration"), str(msg))]

    if not mime_type_is_defined(APP_GRAMPS):
        error += [(_("Configuration error:"),
                    _("A definition for the MIME-type %s could not "
                      "be found \n\n Possibly the installation of Gramps "
                      "was incomplete. Make sure the MIME-types "
                      "of Gramps are properly installed.")
                    % APP_GRAMPS)]

    # we start with parsing the arguments to determine if we have a cli or a
    # gui session

    if "-v" in sys.argv or "--version" in sys.argv:
        show_settings()
        return error

    from .cli.argparser import ArgParser
    argv_copy = sys.argv[:]
    argpars = ArgParser(argv_copy)

    # if in safe mode we should point the db dir back to the original dir.
    # It is ok to import config here, 'Defaults' command had its chance...
    if 'SAFEMODE' in os.environ:
        config.set('database.path', os.path.join(ORIG_HOME_DIR, 'grampsdb'))

    # On windows the fontconfig handler is a better choice
    if(win() and ('PANGOCAIRO_BACKEND' not in os.environ)):
        os.environ['PANGOCAIRO_BACKEND'] = "fontconfig"

    # Calls to LOG must be after setup_logging() and ArgParser()
    LOG = logging.getLogger(".locale")
    LOG.debug("Encoding: %s", glocale.encoding)
    LOG.debug("Translating Gramps to %s", glocale.language[0])
    LOG.debug("Collation Locale: %s", glocale.collation)
    LOG.debug("Date/Time Locale: %s", glocale.calendar)
    LOG.debug("Currency Locale: %s", glocale.currency)
    LOG.debug("Number-format Locale: %s", glocale.numeric)

    if 'LANG' in os.environ:
        LOG.debug('Using LANG: %s' %
                         get_env_var('LANG'))
    else:
        LOG.debug('environment: LANG is not defined')
    if 'LANGUAGE' in os.environ:
        LOG.debug('Using LANGUAGE: %s' %
                         get_env_var('LANGUAGE'))
    else:
        LOG.debug('environment: LANGUAGE is not defined')

    if argpars.need_gui():
        LOG.debug("A GUI is needed, set it up")
        try:
            from .gui.grampsgui import startgramps
            # no DISPLAY is a RuntimeError in an older pygtk (e.g. F14's 2.17)
        except RuntimeError as msg:
            error += [(_("Configuration error:"), str(msg))]
            return error
        startgramps(error, argpars)
    else:
        # CLI use of Gramps
        argpars.print_help()
        argpars.print_usage()
        from .cli.grampscli import startcli
        startcli(error, argpars)

def main():
    if 'GRAMPS_RESOURCES' not in os.environ:
        resource_path, filename = os.path.split(os.path.abspath(__file__))
        resource_path, dirname = os.path.split(resource_path)
        os.environ['GRAMPS_RESOURCES'] = resource_path
    errors = run()
    if errors and isinstance(errors, list):
        for error in errors:
            logging.warning(error[0] + error[1])

if __name__ == '__main__':
    main()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# test/LosHawlos_bsddbtest.py

from bsddb3 import dbshelve, db
import os
import sys

sys.path.append('../gramps')
import gramps.gen.const as const

print("Test that db.DBEnv().open() works")
env_name = os.path.expanduser(os.path.join(const.HOME_DIR, "test"))
if not os.path.isdir(env_name):
       os.mkdir(env_name)

env = db.DBEnv()
env.set_cachesize(0,0x2000000)
env.set_lk_max_locks(25000)
env.set_lk_max_objects(25000)
# env.set_flags(db.DB_LOG_AUTOREMOVE,1)
"""
BSDDB change log settings using new method with renamed attributes
"""
autoremove_flag = None
autoremove_method = None
for flag in ["DB_LOG_AUTO_REMOVE", "DB_LOG_AUTOREMOVE"]:
    if hasattr(db, flag):
        autoremove_flag = getattr(db, flag)
        break
for method in ["log_set_config", "set_flags"]:
    if hasattr(env, method):
        autoremove_method = getattr(env, method)
        break
if autoremove_method and autoremove_flag:
    autoremove_method(autoremove_flag, 1)
else:
    print("Failed to set autoremove flag")
env_flags = db.DB_CREATE|db.DB_RECOVER|db.DB_PRIVATE|\
            db.DB_INIT_MPOOL|db.DB_INIT_LOCK|\
            db.DB_INIT_LOG|db.DB_INIT_TXN|db.DB_THREAD
try:
    env.open(env_name,env_flags)
except db.DBRunRecoveryError as e:
    print("Exception: ")
    print(e)
    env.remove(env_name)
    env.open(env_name,env_flags)
print("OK")



# encoding:utf-8
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2011-2016 Serge Noiraud
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#------------------------------------------------------------------------
#
# Geography view
#
#------------------------------------------------------------------------

# pylint: disable=bad-whitespace
# pylint: disable=bad-whitespace

MODULE_VERSION = "5.1"

from gi import Repository
from gramps.gen.const import GRAMPS_LOCALE as glocale
from gramps.gen.plug._pluginreg import register, VIEW, STABLE #, END, START
_ = glocale.translation.gettext

#-------------------------------------------------------------------------
#
# set up logging
#
#-------------------------------------------------------------------------
import logging
_LOG = logging.getLogger("Geography")

# Attempting to import OsmGpsMap gives an error dialog if OsmGpsMap is not
# available so test first and log just a warning to the console instead.
OSMGPSMAP = False
REPOSITORY = Repository.get_default()
if REPOSITORY.enumerate_versions("OsmGpsMap"):
    try:
        # current osmgpsmap support GTK3
        import gi
        gi.require_version('OsmGpsMap', '1.0')
        from gi.repository import OsmGpsMap as osmgpsmap
        OSMGPSMAP = True
    except:
        pass

if not OSMGPSMAP:
    from gramps.gen.config import config
    if not config.get('interface.ignore-osmgpsmap'):
        from gramps.gen.constfunc import has_display
        if has_display():
            from gramps.gui.dialog import MessageHideDialog
            from gramps.gen.const import URL_WIKISTRING
            TITLE = _("OsmGpsMap module not loaded.")
            MESSAGE = _("Geography functionality will not be available.\n"
                       "Try to install:\n"
                       " gir1.2-osmgpsmap-1.0 (debian, ubuntu, ...)\n"
                       " osm-gps-map-gobject-1.0.1 for fedora, ...\n"
                       " typelib-1_0-OsmGpsMap-1_0 for openSuse\n"
                       " ...\n"
                       "To build it for Gramps see the Wiki (<F1>)\n"
                       " and search for 'build from source'")
            if uistate:
                MessageHideDialog(TITLE, MESSAGE,
                                  'interface.ignore-osmgpsmap',
                                  parent=uistate.window)
            else:
                MessageHideDialog(TITLE, MESSAGE,
                                  'interface.ignore-osmgpsmap')
else:
    # Load the view only if osmgpsmap library is present.
    register(VIEW,
             id = 'geo1',
             name = _("All known places for one Person"),
             description = _("A view showing the places visited by "
                             "one person during his life."),
             version = '1.0',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geoperson.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoPerson',
             #order = START,
             stock_icon = 'geo-show-person',
      )

    register(VIEW,
             id = 'geo2',
             name = _("All known places for one Family"),
             description = _("A view showing the places visited by "
                             "one family during all their life."),
             version = '1.0',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geofamily.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoFamily',
             #order = START,
             stock_icon = 'geo-show-family',
      )

    register(VIEW,
             id = 'geo3',
             name = _("Every residence or move for a person "
                      "and any descendants"),
             description = _("A view showing all the places visited by "
                             "all persons during their life."
                             "\nThis is for a person and any descendant."
                             "\nYou can see the dates corresponding to "
                             "the period."),
             version = '1.0',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geomoves.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoMoves',
             #order = START,
             stock_icon = 'geo-show-family',
      )

    register(VIEW,
             id = 'geo4',
             name = _("Have these two families been able to meet?"),
             description = _("A view showing the places visited by "
                             "all family's members during their life: "
                             "have these two people been able to meet?"),
             version = '1.0.1',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geofamclose.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoFamClose',
             #order = START,
             stock_icon = 'geo-show-family',
      )

    register(VIEW,
             id = 'geo5',
             name = _("Have they been able to meet?"),
             description = _("A view showing the places visited by "
                             "two persons during their life: "
                             "have these two people been able to meet?"),
             version = '1.0.1',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geoclose.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoClose',
             #order = START,
             stock_icon = 'gramps-relation',
      )

    register(VIEW,
             id = 'geo6',
             name = _("All known Places"),
             description = _("A view showing all places of the database."),
             version = '1.0',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geoplaces.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoPlaces',
             #order = START,
             stock_icon = 'geo-show-place',
      )

    register(VIEW,
             id = 'geo7',
             name = _("All places related to Events"),
             description = _("A view showing all the event "
                             "places of the database."),
             version = '1.0',
             gramps_target_version = MODULE_VERSION,
             status = STABLE,
             fname = 'geoevents.py',
             authors = ["Serge Noiraud"],
             authors_email = [""],
             category = ("Geography", _("Geography")),
             viewclass = 'GeoEvents',
             #order = START,
             stock_icon = 'geo-show-event',
      )




#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2014  Nick Hall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#

#-------------------------------------------------------------------------
#
# Gramps classes
#
#-------------------------------------------------------------------------
from .attrembedlist import AttrEmbedList

#-------------------------------------------------------------------------
#
# MediaAttrEmbedList
#
#-------------------------------------------------------------------------
class MediaAttrEmbedList(AttrEmbedList):

    def __init__(self, dbstate, uistate, track, data):
        AttrEmbedList.__init__(self, dbstate, uistate, track, data)

    def get_editor(self):
        from .. import EditAttribute
        return EditAttribute

    def get_user_values(self):
        return self.dbstate.db.get_media_attribute_types()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#-------------------------------------------------------------------------
#
# Standard Python modules
#
#-------------------------------------------------------------------------
from ...const import GRAMPS_LOCALE as glocale
_ = glocale.translation.gettext

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
from . import Rule

#-------------------------------------------------------------------------
#
# HasSource
#
#-------------------------------------------------------------------------
class HasSourceBase(Rule):
    """Rule that checks for a source with a particular value"""


    labels = [ 'Title:',
                    'Author:',
                    'Abbreviation:',
                    'Publication:' ]
    name = 'Sources matching parameters'
    description = "Matches sources with particular parameters"
    category = _('Citation/source filters')
    allow_regex = True

    def apply(self,db,source):
        if not self.match_substring(0,source.get_title()):
            return False

        if not self.match_substring(1,source.get_author()):
            return False

        if not self.match_substring(2,source.get_abbreviation()):
            return False

        if not self.match_substring(3,source.get_publication_info()):
            return False

        return True



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2003-2005  Donald N. Allingham
# Copyright (C) 2008       Brian G. Matherly
# Copyright (C) 2018       Robin van der Vliet
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
"""
Specific classes for relationships.
"""

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------

from gramps.gen.lib import Person
import gramps.gen.relationship

#-------------------------------------------------------------------------
#
#
#
#-------------------------------------------------------------------------

_ordinal_level = [ "",
    "eerste", "tweede", "derde", "vierde", "vijfde",
    "zesde", "zevende", "achtste", "negende", "tiende",
    "elfde", "twaalfde", "dertiende", "veertiende", "vijftiende",
    "zestiende", "zeventiende", "achttiende", "negentiende", "twintigste",
    "eenentwintigste", "tweeëntwintigste", "drieëntwintigste", "vierentwintigste", "vijfentwintigste",
    "zesentwintigste", "zevenentwintigste", "achtentwintigste", "negenentwintigste", "dertigste",
    "eenendertigste", "tweeëndertigste", "drieëndertigste", "vierendertigste", "vijfendertigste",
    "zesendertigste", "zevenendertigste", "achtendertigste", "negenendertigste", "veertigste",
    "eenenveertigste", "tweeënveertigste", "drieënveertigste", "vierenveertigste", "vijfenveertigste",
    "zesenveertigste", "zevenenveertigste", "achtenveertigste", "negenenveertigste", "vijftigste" ]

_removed_level = [ "", "",
    "groot", "overgroot", "betovergroot", "oud", "oudgroot",
    "oudovergroot", "oudbetovergroot", "stam", "stamgroot", "stamovergroot",
    "stambetovergroot", "stamoud", "stamoudgroot", "stamoudovergroot", "stamoudbetovergroot",
    "edel", "edelgroot", "edelovergroot", "edelbetovergroot", "edeloud",
    "edeloudgroot", "edeloudovergroot", "edeloudbetovergroot", "edelstam", "edelstamgroot",
    "edelstamovergroot", "edelstambetovergroot", "edelstamoud", "edelstamoudgroot", "edelstamoudovergroot",
    "edelstamoudbetovergroot", "voor", "voorgroot", "voorovergroot", "voorbetovergroot",
    "vooroud", "vooroudgroot", "vooroudovergroot", "vooroudbetovergroot", "voorstam",
    "voorstamgroot", "voorstamovergroot", "voorstambetovergroot", "voorstamoud", "voorstamoudgroot",
    "voorstamoudovergroot", "voorstamoudbetovergroot", "vooredel" ]

_child_level = [ "", "",
    "klein", "achterklein", "achterachterklein" ]

_nibling_level = [ "", "",
    "achter", "achterachter", "achterachterachter" ]

#-------------------------------------------------------------------------
#
#
#
#-------------------------------------------------------------------------

class RelationshipCalculator(gramps.gen.relationship.RelationshipCalculator):
    """
    RelationshipCalculator Class
    """

    #sibling strings
    STEP = "stief"
    HALF = "half"

    INLAW = "aangetrouwde "

    def __init__(self):
        gramps.gen.relationship.RelationshipCalculator.__init__(self)

    def get_parents(self, level):
        if level > len(_removed_level)-1:
            return "verre voorouders (%d generaties)" % level
        elif level > 4:
            return "%souders (%d generaties)" % (_removed_level[level], level)
        else:
            return "%souders" % _removed_level[level]

    def _get_father(self, level, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if level > len(_removed_level)-1:
            return "verre %s%svoorvader (%d generaties)" % (inlaw, step, level)
        elif level > 4:
            return "%s%s%svader (%d generaties)" % (inlaw, step, _removed_level[level], level)
        else:
            return "%s%s%svader" % (inlaw, step, _removed_level[level])

    def _get_son(self, level, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if level > len(_child_level)-1:
            return "verre %s%sachterkleinzoon (%d generaties)" % (inlaw, step, level)
        else:
            return "%s%s%szoon" % (inlaw, step, _child_level[level])

    def _get_mother(self, level, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if level > len(_removed_level)-1:
            return "verre %s%svoormoeder (%d generaties)" % (inlaw, step, level)
        elif level > 4:
            return "%s%s%smoeder (%d generaties)" % (inlaw, step, _removed_level[level], level)
        else:
            return "%s%s%smoeder" % (inlaw, step, _removed_level[level])

    def _get_daughter(self, level, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if level > len(_child_level)-1:
            return "verre %s%sachterkleindochter (%d generaties)" % (inlaw, step, level)
        else:
            return "%s%s%sdochter" % (inlaw, step, _child_level[level])

    def _get_parent_unknown(self, level, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if level > len(_removed_level)-1:
            return "verre %s%svoorouder (%d generaties)" % (inlaw, step, level)
        elif level > 4:
            return "%s%s%souder (%d generaties)" % (inlaw, step, _removed_level[level], level)
        else:
            return "%s%s%souder" % (inlaw, step, _removed_level[level])

    def _get_child_unknown(self, level, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if inlaw == "aangetrouwde ":
            #The word "kind" is grammatically neuter, so it has a different adjective.
            inlaw = "aangetrouwd "

        if level > len(_child_level)-1:
            return "ver %s%sachterkleinkind (%d generaties)" % (inlaw, step, level)
        else:
            return "%s%s%skind" % (inlaw, step, _child_level[level])

    def _get_aunt(self, level, removed, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if removed == 1 and level < len(_removed_level):
            if level > 4:
                return "%s%s%stante (%d generaties)" % (inlaw, step, _removed_level[level], level)
            else:
                return "%s%s%stante" % (inlaw, step, _removed_level[level])
        elif removed == 1:
            return "verre %s%stante (%d generaties)" % (inlaw, step, level)
        elif level > len(_removed_level)-1 and removed > len(_ordinal_level)-1:
            return "verre %s%stante (%d generaties, %d graden)" % (inlaw, step,
                                                            level, removed)
        elif level > len(_removed_level)-1:
            return "verre %s%stante van de %s graad (%d generaties)" % (inlaw,
                                        step, _ordinal_level[removed], level)
        else:
            if level > 4:
                return "%s%s%stante (%d generaties)" % (inlaw, step, _removed_level[level], level) \
                        + " " + _ordinal_level[removed] + " graad"
            else:
                return "%s%s%stante" % (inlaw, step, _removed_level[level]) \
                        + " " + _ordinal_level[removed] + " graad"

    def _get_uncle(self, level, removed, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if removed == 1 and level < len(_removed_level):
            if level > 4:
                return "%s%s%soom (%d generaties)" % (inlaw, step, _removed_level[level], level)
            else:
                return "%s%s%soom" % (inlaw, step, _removed_level[level])
        elif removed == 1:
            return "verre %s%soom (%d generaties)" % (inlaw, step, level)
        elif level > len(_removed_level)-1 and removed > len(_ordinal_level)-1:
            return "verre %s%soom (%d generaties, %d graden)" % (inlaw, step,
                                                            level, removed)
        elif level > len(_removed_level)-1:
            return "verre %s%soom van de %s graad (%d generaties)" % (inlaw,
                                        step, _ordinal_level[removed], level)
        else:
            if level > 4:
                return "%s%s%soom (%d generaties)" % (inlaw, step, _removed_level[level], level) \
                        + " " + _ordinal_level[removed] + " graad"
            else:
                return "%s%s%soom" % (inlaw, step, _removed_level[level]) \
                        + " " + _ordinal_level[removed] + " graad"

    def _get_sibling(self, level, step="", inlaw=""):
        """overwrite of English method to return unknown gender sibling
        """
        assert(level == 1)
        return self._get_male_cousin(0, step=step, inlaw=inlaw) + " of " \
                + self._get_female_cousin(0, step=step, inlaw=inlaw)

    def _get_nephew(self, level, removed=1, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if removed == 1 and level < len(_nibling_level):
            return "%s%s%sneef" % (inlaw, step, _nibling_level[level])
        elif removed == 1:
            return "verre %s%sneef (%d generaties)" % (inlaw, step, level)
        elif level > len(_nibling_level)-1 and removed > len(_ordinal_level) -1:
            return "verre %s%sneef (%d generaties, %d graden)" % (inlaw, step,
                                                                level, removed)
        elif level > len(_nibling_level)-1:
            return "verre %s%sneef van de %s graad (%d generaties)" % (inlaw, step,
                                                _ordinal_level[removed], level)
        else:
            return "%s%s%sneef" % (inlaw, step, _nibling_level[level]) \
                        + " " + _ordinal_level[removed] + " graad"

    def _get_niece(self, level, removed=1, step="", inlaw=""):
        """Internal Dutch method to create relation string
        """
        if removed == 1 and level < len(_nibling_level):
            return "%s%s%snicht" % (inlaw, step, _nibling_level[level])
        elif removed == 1:
            return "verre %s%snicht (%d generaties)" % (inlaw, step, level)
        elif level > len(_nibling_level)-1 and removed > len(_ordinal_level) -1:
            return "verre %s%snicht (%d generaties, %d graden)" % (inlaw, step,
                                                                level, removed)
        elif level > len(_nibling_level)-1:
            return "verre %s%snicht van de %s graad (%d generaties)" % (inlaw, step,
                                                _ordinal_level[removed], level)
        else:
            return "%s%s%snicht" % (inlaw, step, _nibling_level[level]) \
                        + " " + _ordinal_level[removed] + " graad"

    def _get_male_cousin(self, removed, step="", inlaw=""):
        """Specific Dutch thing, the nieces/nephews on same level are called
            going sideways in a branch as the nieces/newphews going downward
            from your brother/sisters. This used to be called "kozijn"
        """
        removed -= 1
        if removed > len(_ordinal_level)-1:
            return "verre %s%sneef (kozijn, %d graden)" % (inlaw, step,
                                                           removed)
        elif removed == 0:
            return "%s%sbroer" % (inlaw, step)
        else:
            return "%s%sneef (kozijn)" % (inlaw, step) \
                        + " " + _ordinal_level[removed] + " graad"

    def _get_female_cousin(self, removed, step="", inlaw=""):
        """Specific Dutch thing, the nieces/nephews on same level are called
            going sideways in a branch as the nieces/newphews going downward
            from your brother/sisters. This used to be called "kozijn"
        """
        removed -= 1
        if removed > len(_ordinal_level)-1:
            return "verre %s%snicht (kozijn, %d graden)" % (inlaw, step,
                                                           removed)
        elif removed == 0:
            return "%s%szus" % (inlaw, step)
        else:
            return "%s%snicht (kozijn)" % (inlaw, step) \
                        + " " + _ordinal_level[removed] + " graad"

    def get_single_relationship_string(self, Ga, Gb, gender_a, gender_b,
                                       reltocommon_a, reltocommon_b,
                                       only_birth=True,
                                       in_law_a=False, in_law_b=False):
        """
        Return a string representing the relationship between the two people,
        see english method, eg b is father of a
        """
        if only_birth:
            step = ""
        else:
            step = self.STEP

        if in_law_a or in_law_b :
            inlaw = self.INLAW
        else:
            inlaw = ""

        rel_str = "verre %s%sfamilie" % (inlaw, step)

        if Gb == 0:
            #b is ancestor
            if Ga == 0:
                rel_str = "zelfde persoon"
            elif Ga == 1 and inlaw and not step:
                if gender_b == Person.MALE:
                    rel_str = "schoonvader"
                elif gender_b == Person.FEMALE:
                    rel_str = "schoonmoeder"
                else:
                    rel_str = "schoonouder"
            elif gender_b == Person.MALE:
                rel_str = self._get_father(Ga, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_mother(Ga, step, inlaw)
            else:
                rel_str = self._get_parent_unknown(Ga, step, inlaw)
        elif Ga == 0:
            #a is descendant
            if Gb == 1 and inlaw and not step:
                if gender_b == Person.MALE:
                    rel_str = "schoonzoon"
                elif gender_b == Person.FEMALE:
                    rel_str = "schoondochter"
                else:
                    rel_str = "schoonkind"
            elif Gb == 1 and inlaw and step:
                #inlaw stepchild
                if gender_b == Person.MALE:
                    rel_str = "aangetrouwde stiefzoon"
                elif gender_b == Person.FEMALE:
                    rel_str = "aangetrouwde stiefdochter"
                else:
                    rel_str = "aangetrouwd stiefkind"
            elif gender_b == Person.MALE:
                rel_str = self._get_son(Gb, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_daughter(Gb, step, inlaw)
            else:
                rel_str = self._get_child_unknown(Gb, step, inlaw)
        elif Ga > Gb:
            #b is higher in the branch, in english uncle/aunt or
            #cousin up, in Dutch always 'oom/tante'
            if gender_b == Person.MALE:
                rel_str = self._get_uncle(Ga - Gb, Gb, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_aunt(Ga - Gb, Gb, step, inlaw)
            else:
                rel_str = self._get_uncle(Ga - Gb, Gb, step, inlaw) + " of " \
                        + self._get_aunt(Ga - Gb, Gb, step, inlaw)
        elif Ga < Gb:
            #b is lower in the branch, in english niece/nephew or
            #cousin down, in Dutch always 'neef/nicht'
            if gender_b == Person.MALE:
                rel_str = self._get_nephew(Gb - Ga, Ga, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_niece(Gb - Ga, Ga, step, inlaw)
            else:
                rel_str = self._get_nephew(Gb - Ga, Ga, step, inlaw) + " of " \
                        + self._get_niece(Gb - Ga, Ga, step, inlaw)
        else:
            # people on the same level Ga == Gb
            if gender_b == Person.MALE:
                rel_str = self._get_male_cousin(Ga, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_female_cousin(Ga, step, inlaw)
            else:
                rel_str = self._get_male_cousin(Ga, step, inlaw) + " of " \
                        + self._get_female_cousin(Ga, step, inlaw)

        return rel_str

    def get_sibling_relationship_string(self, sib_type, gender_a, gender_b,
                                        in_law_a=False, in_law_b=False):
        """
        Determine the string giving the relation between two siblings of
        type sib_type.
        Eg: b is the brother of a
        Here 'brother' is the string we need to determine
        This method gives more details about siblings than
        get_single_relationship_string can do.

        .. warning:: DON'T TRANSLATE THIS PROCEDURE IF LOGIC IS EQUAL IN YOUR
                     LANGUAGE, AND SAME METHODS EXIST (get_uncle, get_aunt,
                     get_sibling)
        """
        if sib_type == self.NORM_SIB or sib_type == self.UNKNOWN_SIB:
            typestr = ""
        elif sib_type == self.HALF_SIB_FATHER \
                or sib_type == self.HALF_SIB_MOTHER:
            typestr = self.HALF
        elif sib_type == self.STEP_SIB:
            typestr = self.STEP

        if in_law_a or in_law_b :
            inlaw = self.INLAW
        else:
            inlaw = ""

        if inlaw and not typestr:
            if gender_b == Person.MALE:
                rel_str = "schoonbroer"
            elif gender_b == Person.FEMALE:
                rel_str = "schoonzus"
            else:
                rel_str = "schoonbroer of -zus"
        else:
            if gender_b == Person.MALE:
                rel_str = self._get_male_cousin(1, typestr, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_female_cousin(1, typestr, inlaw)
            else:
                rel_str = self._get_male_cousin(1, typestr, inlaw) + " of " \
                        + self._get_female_cousin(1, typestr, inlaw)
        return rel_str


if __name__ == "__main__":
    # Test function. Call it as follows from the command line (so as to find
    #        imported modules):
    #    export PYTHONPATH=/path/to/gramps/src
    #    python src/plugins/rel/rel_nl.py

    """TRANSLATORS, copy this if statement at the bottom of your
        rel_xx.py module, and test your work with:
        python src/plugins/rel/rel_xx.py
    """
    from gramps.gen.relationship import test
    RC = RelationshipCalculator()
    test(RC, True)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2013      Nick Hall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
from .. import Rule
from ....const import GRAMPS_LOCALE as glocale
_ = glocale.translation.gettext

#-------------------------------------------------------------------------
#
# HasDayOfWeek
#
#-------------------------------------------------------------------------
class HasDayOfWeek(Rule):
    """Rule that matches an event occurring on a particular day of the week."""

    labels = [ _('Day of Week:') ]
    name = _('Events occurring on a particular day of the week')
    description = _('Matches events occurring on a particular day of the week')
    category = _('General filters')

    def apply(self, db, event):
        if not self.list[0]:
            return False
        else:
            dow = event.get_date_object().get_dow()
            return dow == int(self.list[0])



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2014-2017  Nick Hall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Class handling displaying of places.
"""

#---------------------------------------------------------------
#
# Python imports
#
#---------------------------------------------------------------
import os
import xml.dom.minidom

#-------------------------------------------------------------------------
#
# Gramps modules
#
#-------------------------------------------------------------------------
from ..const import PLACE_FORMATS, GRAMPS_LOCALE as glocale
_ = glocale.translation.gettext
from ..config import config
from ..utils.location import get_location_list
from ..lib import PlaceType

#-------------------------------------------------------------------------
#
# PlaceFormat class
#
#-------------------------------------------------------------------------
class PlaceFormat:
    def __init__(self, name, levels, language, street, reverse):
        self.name = name
        self.levels = levels
        self.language = language
        self.street = street
        self.reverse = reverse


#-------------------------------------------------------------------------
#
# PlaceDisplay class
#
#-------------------------------------------------------------------------
class PlaceDisplay:

    def __init__(self):
        self.place_formats = []
        self.default_format = config.get('preferences.place-format')
        if os.path.exists(PLACE_FORMATS):
            try:
                self.load_formats()
                return
            except BaseException:
                print(_("Error in '%s' file: cannot load.") % PLACE_FORMATS)
        pf = PlaceFormat(_('Full'), ':', '', 0, False)
        self.place_formats.append(pf)

    def display_event(self, db, event, fmt=-1):
        if not event:
            return ""
        place_handle = event.get_place_handle()
        if place_handle:
            place = db.get_place_from_handle(place_handle)
            return self.display(db, place, event.get_date_object(), fmt)
        else:
            return ""

    def display(self, db, place, date=None, fmt=-1):
        if not place:
            return ""
        if not config.get('preferences.place-auto'):
            return place.title
        else:
            if fmt == -1:
                fmt = config.get('preferences.place-format')
            pf = self.place_formats[fmt]
            lang = pf.language
            all_places = get_location_list(db, place, date, lang)

            # Apply format string to place list
            index = _find_populated_place(all_places)
            places = []
            for slice in pf.levels.split(','):
                parts = slice.split(':')
                if len(parts) == 1:
                    offset = _get_offset(parts[0], index)
                    if offset is not None:
                        try:
                            places.append(all_places[offset])
                        except IndexError:
                            pass
                elif len(parts) == 2:
                    start = _get_offset(parts[0], index)
                    end = _get_offset(parts[1], index)
                    if start is None:
                        places.extend(all_places[:end])
                    elif end is None:
                        places.extend(all_places[start:])
                    else:
                        places.extend(all_places[start:end])

            if pf.street:
                types = [item[1] for item in places]
                try:
                    idx = types.index(PlaceType.NUMBER)
                except ValueError:
                    idx = None
                if idx is not None and len(places) > idx+1:
                    if pf.street == 1:
                        combined = (places[idx][0] + ' ' + places[idx+1][0],
                                    places[idx+1][1])
                    else:
                        combined = (places[idx+1][0] + ' ' + places[idx][0],
                                    places[idx+1][1])
                    places = places[:idx] + [combined] + places[idx+2:]

            names = [item[0] for item in places]
            if pf.reverse:
                names.reverse()

            # TODO for Arabic, should the next line's comma be translated?
            return ", ".join(names)

    def get_formats(self):
        return self.place_formats

    def set_formats(self, formats):
        self.place_formats = formats

    def load_formats(self):
        dom = xml.dom.minidom.parse(PLACE_FORMATS)
        top = dom.getElementsByTagName('place_formats')

        for fmt in top[0].getElementsByTagName('format'):
            name = fmt.attributes['name'].value
            levels = fmt.attributes['levels'].value
            language = fmt.attributes['language'].value
            street = int(fmt.attributes['street'].value)
            reverse = fmt.attributes['reverse'].value == 'True'
            pf = PlaceFormat(name, levels, language, street, reverse)
            self.place_formats.append(pf)

        dom.unlink()

    def save_formats(self):
        doc = xml.dom.minidom.Document()
        place_formats = doc.createElement('place_formats')
        doc.appendChild(place_formats)
        for fmt in self.place_formats:
            node = doc.createElement('format')
            place_formats.appendChild(node)
            node.setAttribute('name', fmt.name)
            node.setAttribute('levels', fmt.levels)
            node.setAttribute('language', fmt.language)
            node.setAttribute('street', str(fmt.street))
            node.setAttribute('reverse', str(fmt.reverse))
        with open(PLACE_FORMATS, 'w', encoding='utf-8') as f_d:
            doc.writexml(f_d, addindent='  ', newl='\n', encoding='utf-8')


def _get_offset(value, index):
    if index is not None and value.startswith('p'):
        try:
            offset = int(value[1:])
        except ValueError:
            offset = 0
        offset += index
    else:
        try:
            offset = int(value)
        except ValueError:
            offset = None
    return offset

def _find_populated_place(places):
    populated_place = None
    for index, item in enumerate(places):
        if int(item[1]) in [PlaceType.HAMLET, PlaceType.VILLAGE,
                            PlaceType.TOWN, PlaceType.CITY]:
            populated_place = index
    return populated_place

displayer = PlaceDisplay()
